@@ -262,6 +262,8 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 			w = int(round(w * scale))
 			h = int(round(h * scale))
 		if parts[-1] == "empty":
+			if wx.VERSION < (3, ):
+				use_mask = True
 			if use_mask and sys.platform == "win32":
 				bmp = wx.EmptyBitmap(w, h)
 				bmp.SetMaskColour(wx.Colour(0, 0, 0))
@@ -323,7 +325,7 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 							# Try original size
 							parts[-1] = oname
 				if (sys.platform not in ("darwin", "win32") and
-					parts[-1].startswith(appname)):
+					parts[-1].startswith(appname.lower())):
 					# Search /usr/share/icons on Linux first
 					path = get_data_path(os.path.join(parts[-2], "apps", parts[-1]) + ".png")
 				if not path:
@@ -414,6 +416,8 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 					if not bmp.IsOk():
 						path = None
 			if not path:
+				from log import safe_print
+				safe_print("Warning: Missing bitmap '%s'" % name)
 				img = wx.EmptyImage(w, h)
 				img.SetMaskColour(0, 0, 0)
 				img.InitAlpha()
@@ -1791,7 +1795,7 @@ def writecfg(which="user", worker=None, module=None, options=()):
 _init_testcharts()
 runtype = runtimeconfig(pyfile)
 
-if isapp and not os.getenv("SSL_CERT_FILE"):
+if sys.platform in ("darwin", "win32") and not os.getenv("SSL_CERT_FILE"):
 	# Use our bundled CA file
 	# https://github.com/certifi/python-certifi/blob/master/certifi/cacert.pem
 	cafile = get_data_path("cacert.pem")
