@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import with_statement
-import glob
 import os
 import re
 import shutil
@@ -12,13 +11,13 @@ if sys.platform == "win32":
 
 from argyll_cgats import cal_to_fake_profile
 from argyll_names import video_encodings
-from config import (get_data_path, get_verified_path, getcfg, geticon, hascfg,
-					profile_ext, setcfg)
+from config import (defaults, get_data_path, get_verified_path, getcfg,
+					geticon, hascfg, profile_ext, setcfg)
 from log import safe_print
 from meta import name as appname, version
 from options import debug
 from util_decimal import stripzeros
-from util_os import islink, readlink, waccess
+from util_os import islink, readlink, safe_glob, waccess
 from util_str import safe_unicode, strtr
 from worker import (Error, Info, UnloggedInfo, get_current_profile_path,
 					show_result_dialog)
@@ -596,7 +595,7 @@ class LUT3DFrame(BaseFrame):
 								break
 					else:
 						# Assume OS X
-						volumes = ["/"] + glob.glob("/Volumes/*")
+						volumes = ["/"] + safe_glob("/Volumes/*")
 						for volume in volumes:
 							lut_dir = os.path.join(volume, "Library",
 												   "Application Support",
@@ -1382,12 +1381,9 @@ class LUT3DFrame(BaseFrame):
 		# Shared with main window
 		self.lut3d_update_trc_controls()
 		self.lut3d_rendering_intent_ctrl.SetSelection(self.rendering_intents_ba[getcfg("3dlut.rendering_intent")])
-		format = getcfg("3dlut.format")
-		if format == "madVR" and self.worker.argyll_version < [1, 6]:
-			# MadVR only available with Argyll 1.6+, fall back to IRIDAS .cube
-			format = "cube"
-			setcfg("3dlut.format", format)
-		self.lut3d_format_ctrl.SetSelection(self.lut3d_formats_ba[getcfg("3dlut.format")])
+		# MadVR only available with Argyll 1.6+, fall back to default
+		self.lut3d_format_ctrl.SetSelection(self.lut3d_formats_ba.get(getcfg("3dlut.format"),
+																	  self.lut3d_formats_ba[defaults["3dlut.format"]]))
 		self.lut3d_hdr_display_ctrl.SetSelection(getcfg("3dlut.hdr_display"))
 		self.lut3d_size_ctrl.SetSelection(self.lut3d_size_ba[getcfg("3dlut.size")])
 		self.lut3d_enable_size_controls()
@@ -1435,22 +1431,16 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_hdr_update_system_gamma()
 		# Content colorspace (currently only used for SMPTE 2084)
 		content_colors = []
-		for color in ("white", "red", "green", "blue"):
+		for color in ("red", "green", "blue", "white"):
 			for coord in "xy":
 				v = getcfg("3dlut.content.colorspace.%s.%s" % (color, coord))
 				getattr(self, "lut3d_content_colorspace_%s_%s" %
 							  (color, coord)).SetValue(v)
 				content_colors.append(round(v, 4))
-		for i, rgb_space in enumerate(self.lut3d_content_colorspace_names):
-			rgb_space = colormath.get_rgb_space(rgb_space)
-			wx, wy = (round(v, 4) for v in
-					  colormath.XYZ2xyY(*colormath.get_whitepoint(rgb_space[1]))[:2])
-			colors = [wx, wy]
-			for primary in rgb_space[2:5]:
-				for j in xrange(2):
-					colors.append(round(primary[j], 4))
-			if colors == content_colors:
-				break
+		rgb_space_name = colormath.find_primaries_wp_xy_rgb_space_name(content_colors,
+																	   self.lut3d_content_colorspace_names)
+		if rgb_space_name:
+			i = self.lut3d_content_colorspace_names.index(rgb_space_name)
 		else:
 			i = self.lut3d_content_colorspace_ctrl.Count - 1
 		self.lut3d_content_colorspace_ctrl.SetSelection(i)
