@@ -14,11 +14,11 @@ try:
 except ImportError:
 	pass
 else:
-	if not getattr(sys, "frozen", False):
+	if not getattr(sys, "frozen", False) and not "wx" in sys.modules:
 		wxversion.ensureMinimal("%i.%i" % wx_minversion[:2])
 import wx
 if wx.VERSION < wx_minversion:
-	app = wx.PySimpleApp()
+	app = wx.GetApp() or wx.App(0)
 	result = wx.MessageBox("This application requires a version of wxPython "
 						   "greater than or equal to %s, but your most recent "
 						   "version is %s.\n\n"
@@ -37,9 +37,15 @@ from wx.lib.buttons import GenBitmapTextButton as _GenBitmapTextButton
 from wx.lib import platebtn
 from wx import xrc
 
+from colormath import convert_range
 from util_str import safe_str
 
 
+if not hasattr(wx.Window, "HasFocus"):
+	# wxPython < 3.0
+	wx.Window.HasFocus = lambda self: self.FindFocus() is self
+
+
 if not hasattr(platebtn, "PB_STYLE_TOGGLE"):
 	platebtn.PB_STYLE_TOGGLE = 32
 
@@ -91,9 +97,25 @@ if u"phoenix" in wx.PlatformInfo:
 	wx.VERTICAL_HATCH      = wx.BRUSHSTYLE_VERTICAL_HATCH
 
 	embeddedimage.PyEmbeddedImage.getBitmap = embeddedimage.PyEmbeddedImage.GetBitmap
+
+	def BitmapFromIcon(icon):
+		bmp = wx.Bitmap()
+		bmp.CopyFromIcon(icon)
+		return bmp
+
+	def IconFromBitmap(bmp):
+		icon = wx.Icon()
+		icon.CopyFromBitmap(bmp)
+		return bmp
+
+	wx.BitmapFromIcon = BitmapFromIcon
 	wx.CursorFromImage = wx.Cursor
 	wx.EmptyBitmap = wx.Bitmap
+	wx.EmptyBitmapRGBA = (lambda width, height, red=0, green=0, blue=0, alpha=0:
+						  wx.Bitmap.FromRGBA(width, height, red, green, blue,
+											 alpha))
 	wx.EmptyIcon = wx.Icon
+	wx.IconFromBitmap = wx.Icon
 	wx.ImageFromStream = wx.Image
 	wx.ListCtrl.InsertStringItem = lambda self, index, label: self.InsertItem(index, label)
 	wx.ListCtrl.SetStringItem = lambda self, index, col, label: self.SetItem(index, col, label)
@@ -110,8 +132,31 @@ if u"phoenix" in wx.PlatformInfo:
 	wx.HL_DEFAULT_STYLE = wx.adv.HL_DEFAULT_STYLE
 	wx.HyperlinkCtrl = wx.adv.HyperlinkCtrl
 	wx.HyperlinkCtrlNameStr = wx.adv.HyperlinkCtrlNameStr
+
 	wx.SOUND_ASYNC = wx.adv.SOUND_ASYNC
 	wx.Sound = wx.adv.Sound
+	
+	wx.wxEVT_TASKBAR_BALLOON_CLICK = wx.adv.wxEVT_TASKBAR_BALLOON_CLICK
+	wx.wxEVT_TASKBAR_BALLOON_TIMEOUT = wx.adv.wxEVT_TASKBAR_BALLOON_TIMEOUT
+	wx.wxEVT_TASKBAR_CLICK = wx.adv.wxEVT_TASKBAR_CLICK
+	wx.wxEVT_TASKBAR_LEFT_DCLICK = wx.adv.wxEVT_TASKBAR_LEFT_DCLICK
+	wx.wxEVT_TASKBAR_LEFT_DOWN = wx.adv.wxEVT_TASKBAR_LEFT_DOWN
+	wx.wxEVT_TASKBAR_LEFT_UP = wx.adv.wxEVT_TASKBAR_LEFT_UP
+	wx.wxEVT_TASKBAR_MOVE = wx.adv.wxEVT_TASKBAR_MOVE
+	wx.wxEVT_TASKBAR_RIGHT_DOWN = wx.adv.wxEVT_TASKBAR_RIGHT_DOWN
+	wx.wxEVT_TASKBAR_RIGHT_UP = wx.adv.wxEVT_TASKBAR_RIGHT_UP
+	wx.wxEVT_TASKBAR_RIGHT_DCLICK = wx.adv.wxEVT_TASKBAR_RIGHT_DCLICK
+	
+	wx.EVT_TASKBAR_BALLOON_CLICK = wx.adv.EVT_TASKBAR_BALLOON_CLICK
+	wx.EVT_TASKBAR_BALLOON_TIMEOUT = wx.adv.EVT_TASKBAR_BALLOON_TIMEOUT
+	wx.EVT_TASKBAR_CLICK = wx.adv.EVT_TASKBAR_CLICK
+	wx.EVT_TASKBAR_LEFT_DCLICK = wx.adv.EVT_TASKBAR_LEFT_DCLICK
+	wx.EVT_TASKBAR_LEFT_DOWN = wx.adv.EVT_TASKBAR_LEFT_DOWN
+	wx.EVT_TASKBAR_LEFT_UP = wx.adv.EVT_TASKBAR_LEFT_UP
+	wx.EVT_TASKBAR_MOVE = wx.adv.EVT_TASKBAR_MOVE
+	wx.EVT_TASKBAR_RIGHT_DOWN = wx.adv.EVT_TASKBAR_RIGHT_DOWN
+	wx.EVT_TASKBAR_RIGHT_UP = wx.adv.EVT_TASKBAR_RIGHT_UP
+	wx.EVT_TASKBAR_RIGHT_DCLICK = wx.adv.EVT_TASKBAR_RIGHT_DCLICK
 
 	# Removed items
 
@@ -119,6 +164,11 @@ if u"phoenix" in wx.PlatformInfo:
 	wx.DC.DrawRectangleRect = lambda dc, rect: dc.DrawRectangle(rect)
 	wx.DC.DrawRoundedRectangleRect = lambda dc, rect, radius: dc.DrawRoundedRectangle(rect, radius)
 	wx.DC.EndDrawing = lambda self: None
+	if not hasattr(wx.DC, "SetClippingRect"):
+		wx.DC.SetClippingRect = lambda dc, rect: dc.SetClippingRegion(rect.x,
+																	  rect.y,
+																	  rect.width,
+																	  rect.height)
 
 	wx.IconBundle.AddIconFromFile = lambda file, type=wx.BITMAP_TYPE_ANY: wx.IconBundle.AddIcon(file, type)
 
@@ -152,7 +202,12 @@ if u"phoenix" in wx.PlatformInfo:
 	wx.SystemSettings_GetFont = wx.SystemSettings.GetFont
 	wx.SystemSettings_GetMetric = wx.SystemSettings.GetMetric
 	wx.grid.EVT_GRID_CELL_CHANGE = wx.grid.EVT_GRID_CELL_CHANGED
-	wx.grid.Grid.wxGridSelectRows = wx.grid.Grid.GridSelectRows
+	if hasattr( wx.grid.Grid, "GridSelectRows"):
+		# wxPython Phoenix 4.0.0a
+		wx.grid.Grid.wxGridSelectRows = wx.grid.Grid.GridSelectRows
+	else:
+		# wxPython Phoenix 4.0.0b
+		wx.grid.Grid.wxGridSelectRows = wx.grid.Grid.SelectRows
 	wx.grid.PyGridCellEditor = wx.grid.GridCellEditor
 	wx.grid.PyGridCellRenderer = wx.grid.GridCellRenderer
 	wx.RegionFromBitmapColour = wx.Region
@@ -518,64 +573,88 @@ def SetToolTipString(self, string):
 wx.Window.SetToolTipString = SetToolTipString
 
 
-wx.Sizer._Add = wx.Sizer.Add
+wx._Sizer = wx.Sizer
 
-def SizerAdd(self, *args, **kwargs):
-	from config import get_default_dpi, getcfg
-	scale = getcfg("app.dpi") / get_default_dpi()
-	if scale > 1:
-		if kwargs.get("border"):
-			kwargs["border"] = int(round(kwargs["border"] * scale))
-		if args and isinstance(args[0], tuple):
-			spacer = list(args[0])
-			##print spacer, '->',
-			for i, dimension in enumerate(spacer):
-				if dimension > 0:
-					spacer[i] = int(round(dimension * scale))
-			##print spacer
-			args = (tuple(spacer), ) + args[1:]
-				
-	return self._Add(*args, **kwargs)
+class Sizer(wx._Sizer):
 
-if not u"phoenix" in wx.PlatformInfo:
-	# Doesn't work with Phoenix
-	wx.Sizer.Add = SizerAdd
+	def Add(self, *args, **kwargs):
+		from config import get_default_dpi, getcfg
+		scale = getcfg("app.dpi") / get_default_dpi()
+		if isinstance(args[0], int):
+			args = list(args)
+			index = args.pop(0)
+			args = tuple(args)
+		else:
+			index = None
+		if scale > 1:
+			if kwargs.get("border"):
+				kwargs["border"] = int(round(kwargs["border"] * scale))
+			if args and isinstance(args[0], tuple):
+				spacer = list(args[0])
+				##print spacer, '->',
+				for i, dimension in enumerate(spacer):
+					if dimension > 0:
+						spacer[i] = int(round(dimension * scale))
+				##print spacer
+				args = (tuple(spacer), ) + args[1:]
+		if index is None:
+			return wx._Sizer.Add(self, *args, **kwargs)
+		else:
+			return wx._Sizer.Insert(self, index, *args, **kwargs)
+
+	Insert = Add
+
+wx.Sizer = Sizer
 
 
-	wx._GridSizer = wx.GridSizer
+wx._BoxSizer = wx.BoxSizer
 
-	class GridSizer(wx._GridSizer):
+class BoxSizer(wx._BoxSizer):
 
-		def __init__(self, rows=0, cols=0, vgap=0, hgap=0):
-			if vgap or hgap:
-				from config import get_default_dpi, getcfg
-				scale = getcfg("app.dpi") / get_default_dpi()
-				if scale > 1:
-					##print vgap, hgap, '->',
-					vgap, hgap = [int(round(v * scale)) for v in (vgap, hgap)]
-					##print vgap, hgap
-			super(GridSizer, self).__init__(rows, cols, vgap, hgap)
+	Add = Sizer.__dict__["Add"]
+	Insert = Add
 
-	# TODO: Yet to test if this works with Phoenix
-	wx.GridSizer = GridSizer
+wx.BoxSizer = BoxSizer
 
 
-	wx._FlexGridSizer = wx.FlexGridSizer
+wx._GridSizer = wx.GridSizer
 
-	class FlexGridSizer(wx._FlexGridSizer):
+class GridSizer(wx._GridSizer):
 
-		def __init__(self, rows=0, cols=0, vgap=0, hgap=0):
-			if vgap or hgap:
-				from config import get_default_dpi, getcfg
-				scale = getcfg("app.dpi") / get_default_dpi()
-				if scale > 1:
-					##print vgap, hgap, '->',
-					vgap, hgap = [int(round(v * scale)) for v in (vgap, hgap)]
-					##print vgap, hgap
-			super(FlexGridSizer, self).__init__(rows, cols, vgap, hgap)
+	def __init__(self, rows=0, cols=0, vgap=0, hgap=0):
+		if vgap or hgap:
+			from config import get_default_dpi, getcfg
+			scale = getcfg("app.dpi") / get_default_dpi()
+			if scale > 1:
+				##print vgap, hgap, '->',
+				vgap, hgap = [int(round(v * scale)) for v in (vgap, hgap)]
+				##print vgap, hgap
+		super(GridSizer, self).__init__(rows, cols, vgap, hgap)
 
-	# TODO: Yet to test if this works with Phoenix
-	wx.FlexGridSizer = FlexGridSizer
+	Add = Sizer.__dict__["Add"]
+	Insert = Add
+
+wx.GridSizer = GridSizer
+
+
+wx._FlexGridSizer = wx.FlexGridSizer
+
+class FlexGridSizer(wx._FlexGridSizer):
+
+	def __init__(self, rows=0, cols=0, vgap=0, hgap=0):
+		if vgap or hgap:
+			from config import get_default_dpi, getcfg
+			scale = getcfg("app.dpi") / get_default_dpi()
+			if scale > 1:
+				##print vgap, hgap, '->',
+				vgap, hgap = [int(round(v * scale)) for v in (vgap, hgap)]
+				##print vgap, hgap
+		super(FlexGridSizer, self).__init__(rows, cols, vgap, hgap)
+
+	Add = Sizer.__dict__["Add"]
+	Insert = Add
+
+wx.FlexGridSizer = FlexGridSizer
 
 
 def GridGetSelection(self):
@@ -665,15 +744,93 @@ def get_bitmap_disabled(bitmap):
 	return image.ConvertToBitmap()
 
 
-def get_bitmap_hover(bitmap):
+def get_bitmap_hover(bitmap, ctrl=None):
 	image = bitmap.ConvertToImage()
 	if image.HasMask() and not image.HasAlpha():
 		image.InitAlpha()
+	if sys.platform == "darwin":
+		color = [44, 93, 205]  # Use Mavericks-like color scheme
+	else:
+		color = list(wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT)[:3])
+	R = color[0] / 255.0
+	G = color[1] / 255.0
+	B = color[2] / 255.0
+	# Assume Rec. 709 gamma 2.2 to determine rel. luminance
+	lum = .2126 * R ** 2.2 + .7152 * G ** 2.2 + .0722 * B ** 2.2
+	is_bw = image.IsBW()
+	if ctrl:
+		bgcolor = ctrl.Parent.BackgroundColour
+		if is_bw:
+			# Adjust hilight color by background color if luminance difference
+			# is below threshhold
+			R = bgcolor[0] / 255.0
+			G = bgcolor[1] / 255.0
+			B = bgcolor[2] / 255.0
+			# Assume Rec. 709 gamma 2.2 to determine rel. luminance
+			bglum = .2126 * R ** 2.2 + .7152 * G ** 2.2 + .0722 * B ** 2.2
+			lumdiff = (max(lum, bglum) - min(lum, bglum)) ** (1 / 2.2)
+			if lumdiff < 1.0 / 3.0:
+				if lum > bglum:
+					newmin = bglum ** (1 / 2.2) * 255
+					newmax = 255
+				else:
+					newmin = 0
+					newmax = bglum ** (1 / 2.2) * 153  # 60%
+				for i, v in enumerate(color):
+					color[i] = int(round(convert_range(v, 0, 255, newmin,
+													   newmax)))
 	databuffer = image.GetDataBuffer()
+	alphabuffer = image.GetAlphaBuffer()
+	minv = 256  # Intentionally above max possible value
+	j = 0
+	RGB = {}
 	for i, byte in enumerate(databuffer):
-		if byte > "\0":
-			databuffer[i] = chr(int(round(min(ord(byte) * 1.15, 255))))
-	return image.ConvertToBitmap()
+		RGB[i % 3] = ord(byte)
+		if i % 3 == 2:
+			RGBv = RGB.values()
+			if not is_bw:
+				RGBv_max = max(RGBv)
+				if minv == 256 and alphabuffer[j] > "\x20":
+					# If a pixel is at least 12.5% opaque, check if it's value
+					# is less than or equal the previously seen minimum pixel
+					# value as a crude means to determine if the graphic
+					# contains a black outline.
+					minv = min(RGBv_max, minv)
+			for k in xrange(3):
+				if is_bw:
+					v = color[k]
+				else:
+					v = RGB[k]
+					if minv > 0 and min(RGBv) != RGBv_max:
+						# If the minimum seen at least 12.5% opaque pixel value
+						# is above zero, assume a graphic without black outline
+						# and adjust intensity by applying a power. The hilight
+						# color isn't used.
+						v = (v / 255.0) ** 0.8 * 255
+					else:
+						# If the minimum seen at least 12.5% opaque pixel value
+						# is zero, adjust value by hilight color value, so that
+						# graphics with a black outline get the hilight color
+						# at black.
+						v = convert_range(v, 0, 255, color[k], 255)
+					v = int(round(v))
+				databuffer[i - 2 + k] = chr(v)
+			j += 1
+	if isinstance(ctrl, wx.BitmapButton) and sys.platform == "win32":
+		# wx.BitmapButton draws the focus bitmap over the normal bitmap,
+		# leading to ugly aliased edges. Yuck. Prevent this by setting the
+		# background to the control's background color instead of transparent.
+		bmp = wx.EmptyBitmap(image.Width, image.Height)
+		dc = wx.MemoryDC()
+		dc.SelectObject(bmp)
+		dc.SetBrush(wx.Brush(bgcolor))
+		dc.SetPen(wx.Pen(bgcolor, width=0))
+		dc.DrawRectangle(0, 0, image.Width, image.Height)
+		dc.DrawBitmap(image.ConvertToBitmap(), 0, 0, True)
+		dc.SelectObject(wx.NullBitmap)
+	else:
+		bmp = image.ConvertToBitmap()
+	return bmp
 
 
 def get_bitmap_pressed(bitmap):
@@ -687,7 +844,18 @@ def get_bitmap_pressed(bitmap):
 	return image.ConvertToBitmap()
 
 
-def set_bitmap_labels(btn, disabled=True, focus=True, pressed=True):
+def get_bitmap_focus(bitmap):
+	image = bitmap.ConvertToImage()
+	if image.HasMask() and not image.HasAlpha():
+		image.InitAlpha()
+	databuffer = image.GetDataBuffer()
+	for i, byte in enumerate(databuffer):
+		if byte > "\0":
+			databuffer[i] = chr(int(round((ord(byte) / 255.0) ** 0.8 * 255)))
+	return image.ConvertToBitmap()
+
+
+def set_bitmap_labels(btn, disabled=True, focus=None, pressed=True):
 	bitmap = btn.BitmapLabel
 	if not bitmap.IsOk():
 		size = btn.MinSize
@@ -701,9 +869,14 @@ def set_bitmap_labels(btn, disabled=True, focus=True, pressed=True):
 		btn.SetBitmapDisabled(get_bitmap_disabled(bitmap))
 
 	# Focus/Hover
-	if sys.platform != "darwin" and focus:
+	if sys.platform != "darwin" and focus is not False:
 		# wxMac applies hover state also to disabled buttons...
-		bmp = get_bitmap_hover(bitmap)
+		if focus is None:
+			# Use hover bitmap
+			bmp = get_bitmap_hover(bitmap, btn)
+		else:
+			# Use focus bitmap
+			bmp = get_bitmap_focus(bitmap)
 		btn.SetBitmapFocus(bmp)
 		if hasattr(btn, "SetBitmapCurrent"):
 			# Phoenix
@@ -714,7 +887,10 @@ def set_bitmap_labels(btn, disabled=True, focus=True, pressed=True):
 
 	# Selected
 	if pressed:
-		bmp = get_bitmap_pressed(bitmap)
+		if sys.platform == "darwin":
+			bmp = get_bitmap_hover(bitmap, btn)
+		elif focus is False:
+			bmp = get_bitmap_pressed(bitmap)
 		if hasattr(btn, "SetBitmapPressed"):
 			# Phoenix
 			btn.SetBitmapPressed(bmp)
@@ -1166,7 +1342,7 @@ class PlateButton(platebtn.PlateButton):
 		else:
 			bmp = self._bmp['disable']
 
-		xpos = 16
+		xpos = 12
 		if bmp is not None and bmp.IsOk():
 			bw, bh = bmp.GetSize()
 			ypos = (self.GetSize()[1] - bh) // 2
@@ -1195,7 +1371,10 @@ class PlateButton(platebtn.PlateButton):
 
 		# Calc Object Positions
 		width, height = self.GetSize()
-		tw, th = gc.GetTextExtent(self.Label)
+		# XXX: Using self.GetTextextent instead of gc.GetTextExtent
+		# seems to fix sporadic segfaults with wxPython Phoenix under Windows.
+		# TODO: Figure out why this is the case.
+		tw, th = self.GetTextExtent(self.Label)
 		txt_y = max((height - th) // 2, 1)
 
 		# The background needs some help to look transparent on
@@ -1219,7 +1398,7 @@ class PlateButton(platebtn.PlateButton):
 
 			self.__DrawHighlight(gc, width, height)
 			txt_x = self.__DrawBitmap(gc)
-			t_x = max((width - tw - (txt_x + 2)) // 2, txt_x + 2)
+			t_x = max((width - tw - (txt_x + 8)) // 2, txt_x + 8)
 			gc.DrawText(self.Label, t_x, txt_y)
 			self.__DrawDropArrow(gc, width - 10, (height // 2) - 2)
 
@@ -1233,7 +1412,7 @@ class PlateButton(platebtn.PlateButton):
 		# Draw bitmap and text
 		if self._state['cur'] != platebtn.PLATE_PRESSED or not self.IsEnabled():
 			txt_x = self.__DrawBitmap(gc)
-			t_x = max((width - tw - (txt_x + 2)) // 2, txt_x + 2)
+			t_x = max((width - tw - (txt_x + 8)) // 2, txt_x + 8)
 			gc.DrawText(self.Label, t_x, txt_y)
 			self.__DrawDropArrow(gc, width - 10, (height // 2) - 2)
 
@@ -1256,6 +1435,11 @@ class PlateButton(platebtn.PlateButton):
 		bevt.SetString(self.GetLabel())
 		self.GetEventHandler().ProcessEvent(bevt)
 
+	_PostEvent = __PostEvent
+
+	def Update(self):
+		pass
+
 	Disable = ThemedGenButton.__dict__["Disable"]
 	Enable = ThemedGenButton.__dict__["Enable"]
 	Enabled = ThemedGenButton.__dict__["Enabled"]
@@ -1264,6 +1448,7 @@ class PlateButton(platebtn.PlateButton):
                      lambda self, label: self.SetLabel(label))
 
 if not hasattr(PlateButton, "_SetState"):
+	# wxPython < 3.0
 	PlateButton._SetState = PlateButton.SetState
 
 
